.nolist
include \masm32\include\masm32rt.inc
.686
.xmm
include \masm32\macros\timers.asm

NumBytes=	4096
LOOP_COUNT=	200
UseMemOn=	1
ShowPerK=	1

ShowCycles MACRO arg
LOCAL tmp$, Lset$
  if ShowPerK
	push eax
	fild dword ptr [esp]
	mov dword ptr [esp], 1024	;; one kByte
	fild dword ptr [esp]
	fmul
	mov dword ptr [esp], NumBytes
	fild dword ptr [esp]
	fdiv
	fistp dword ptr [esp]
	pop eax
  endif
  Lset$ CATSTR <arg>, <               >
  Lset$ SUBSTR Lset$, 1, 16
  tmp$ CATSTR <print str$(eax),  9, "cycles per kByte for >, Lset$, <">
;  % echo tmp$
  tmp$
  print hex$(NumBytes), "h bytes ", 40
  mov eax, NumBytes
  shr eax, 10
  .if eax<1024
	print str$(eax), " kB", 41, 13, 10
  .else
	shr eax, 10
	print str$(eax), " MB", 41, 13, 10
  .endif
  
ENDM

UseTheMem MACRO
if UseMemOn
  mov edx, eax
  mov ecx, 0
  .Repeat
	mov byte ptr [edx+ecx+123], 0		; poke a byte in each page
	add ecx, 4096
  .Until ecx>=NumBytes
endif
ENDM

TestH	MACRO
  invoke HeapAlloc, ProHeap, HEAP_GENERATE_EXCEPTIONS or HEAP_NO_SERIALIZE, NumBytes
  .if eax
	UseTheMem
	invoke HeapFree, ProHeap, HEAP_NO_SERIALIZE, eax
  .else
	invoke MessageBox, 0, chr$("HeapAlloc failed"), 0, MB_OK
	exit
  .endif
ENDM

TestV	MACRO
  invoke VirtualAlloc, NULL, NumBytes, MEM_COMMIT, PAGE_READWRITE
  .if eax
	UseTheMem
	invoke VirtualFree, eax, 0, MEM_RELEASE
  .else
	invoke MessageBox, 0, chr$("VirtualAlloc failed"), 0, MB_OK
	exit
  .endif
ENDM

TestG	MACRO
  invoke GlobalAlloc, GMEM_FIXED, NumBytes
  .if eax
	UseTheMem
	invoke GlobalFree, eax
  .else
	invoke MessageBox, 0, chr$("GlobalAlloc failed"), 0, MB_OK
	exit
  .endif
ENDM

.data?
ProHeap	dd ?

.code
Start:
	push 1
	call ShowCpu
	invoke GetProcessHeap
	.if eax==0
		MsgBox 0, chr$("GetProcessHeap failure"), chr$("HeapSpeed:"), MB_OK
		exit
	.endif
	mov ProHeap, eax
	finit

	REPEAT 8
	counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS
		TestH
	counter_end
	ShowCycles HeapAlloc

	counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS
		TestV
	counter_end
	ShowCycles VirtualAlloc

	counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS
		TestG
	counter_end
	ShowCycles GlobalAlloc
	print chr$(13, 10)
	NumBytes=NumBytes*4
	if NumBytes ge 400000h
		LOOP_COUNT=	10
	endif
	ENDM
	print "UseMem="
	print str$(UseMemOn), ", cycles per kByte="
	print str$(ShowPerK)

	getkey
	exit

ShowCpu proc ; mode:DWORD
COMMENT	@ Usage: 
Move this line up:
ShowCpu	PROTO:DWORD
	invoke ShowCpu, 0		; simple, no printing, just returns SSE level
	invoke ShowCpu, 1		; prints the brand string and returns SSE level@
  pushad
  sub esp, 80	; create a buffer for the brand string
  mov edi, esp		; point edi to it
  xor ebp, ebp
  .Repeat
  	lea eax, [ebp+80000002h]
	db 0Fh, 0A2h		; cpuid 80000002h-80000004h
	stosd
	mov eax, ebx
	stosd
	mov eax, ecx
	stosd
	mov eax, edx
	stosd
	inc ebp
  .Until ebp>=3
  push 1
  pop eax
  db 0Fh, 0A2h			; cpuid 1
  xor ebx, ebx					; CpuSSE
  xor esi, esi					; add zero plus the carry flag
  bt edx, 25					; edx bit 25, SSE1
  adc ebx, esi
  bt edx, 26					; edx bit 26, SSE2
  adc ebx, esi
  bt ecx, esi					; ecx bit 0, SSE3
  adc ebx, esi
  bt ecx, 9						; ecx bit 9, SSE4
  adc ebx, esi
  dec dword ptr [esp+4+32+80]				; dec mode in stack
  .if Zero?
	mov edi, esp							; restore pointer to brand string
  	.Repeat				
		.Break .if byte ptr [edi]!=32	; mode was 1, so show a string but skip leading blanks
		inc edi
	.Until 0
	print edi								; CpuBrand
	.if ebx
		print chr$(32, 40, "SSE")		; info on SSE level, 40=(
		print str$(ebx), 41, 13, 10		; 41=)
	.endif
  .endif
  add esp, 80				; discard brand buffer (after printing!)
  mov [esp+32-4], ebx		; move ebx into eax stack position - returns eax to main for further use
  popad
  ret 4
ShowCpu endp
end Start

OPT_WAIT	0
OPT_Tmp2Asm	1
